/* Frazioni Egizie. */
/* Data una frazione 0 < a/b <= 1, la sviluppa come */
/* a/b = 1/d1 - 1/(d1*d2) + 1/(d1*d2*d3) - 1/(d1*d2*d3*d4) ... 1/(d1*d2* ... *dn) */

/* Versione con frazione implementata come struct */

#include <stdio.h>

struct frazione {
	int numeratore;
	unsigned denominatore; 
};

/* restituisce il gcd di a e b calcolato con l'algoritmo di Euclide */
unsigned gcd(unsigned a, unsigned b)
{
	unsigned t;

	while(b) {
		t = b;
		b = a % b;
		a = t;
	}
	return a;
}

void stampafrazione(struct frazione f)
{
	printf("%d",f.numeratore);
	if(f.denominatore > 1)
		printf("/%d",f.denominatore);
}

/* restituisce la forma irriducibile della frazione f */
struct frazione formairriducibile(struct frazione f)
{
	short sign = f.numeratore >= 0 ? 1 : -1;
	unsigned d = gcd(sign * f.numeratore,f.denominatore);
	
	f.numeratore /= (int)d;
	f.denominatore /= d;
	return f;
}


/* Data h/k restituisce la prossima frazione h'/k' da espandere nello sviluppo in frazioni egizie: */
/* d = floor(h/k) */
/* h/k = 1/d - 1/d * (h'/k') */
/* carica nella variabile puntata da pden, il valore di d */ 
struct frazione prossimafrazione(struct frazione f, int *pden)
{
	struct frazione prox;

	*pden = f.denominatore / f.numeratore;
	prox.numeratore = f.denominatore - f.numeratore * *pden;
	prox.denominatore = f.denominatore;
	return formairriducibile(prox);		
}

int main(void)
{
	struct frazione f;
	int den,newden;
	char parity = 0; /* per gestire nello sviluppo l'alternanza +/- del segno  */

	printf("Immetti numeratore e denominatore della frazione:\n");
	scanf("%d %d",&f.numeratore,&den);
	if(den <= 0) {
		printf("Errore di immissione, il denominatore %d non e' > 0\n",den);
		return -1;
	}
	if(f.numeratore > den) {
		printf("Errore di immissione, il denominatore %d non e' maggiore del numeratore %d\n",den,f.numeratore);
		return -1;
	}
	if(f.numeratore <= 0) {
		printf("Errore di immissione, il numeratore %d non e' > 0 \n",f.numeratore);
		return -1;
	}

	f.denominatore = den;
	stampafrazione(f);
	printf(" = ");

	f = formairriducibile(f);
	den = 1;
	while(f.numeratore > 1) {
		f = prossimafrazione(f,&newden);
		printf("1/%d %c ", den *= newden, ++parity % 2 ? '-' : '+');		
	}
	printf("1/%d\n",f.denominatore * den);	

	return 0;
}
